var request = require( 'request' ),
    querystring = require( 'querystring' ),
    async = require( 'async' );

function api_request ( api_url, user_password, options, callback ) {
  var headers = options.headers || {};

  if ( user_password ) {
    var user = user_password[ 0 ],
        password = user_password[ 1 ],
        auth = user + ':' + password;

    auth = new Buffer( auth, 'ascii' ).toString( 'base64' );
    auth = 'Basic ' + auth;
    headers[ 'Authorization' ] = auth;
  }

  if ( options.body ) {
    headers[ 'Content-Type' ] = 'application/json';
  }

  options.headers = headers;
  options.uri = api_url + options.uri;
  request( options, callback );
}

function Ramen ( options ) {
  this.blog = options.blog;
  this.url = options.url;
  this.userpass = null

  if ( options.user && options.password ) {
    this.userpass = [
      options.user,
      options.password
    ];
  }
}

module.exports = Ramen;

Ramen.prototype._query = function ( type, item ) {
  return new Query({
    blog: this.blog,
    url: this.url,
    userpass: this.userpass,
    type: type,
    item: item || null
  });
};

[ 'users', 'authors', 'blogs',
  'posts', 'tags' ].forEach( function ( type ) {
  Ramen.prototype[ type ] = function ( item ) {
    return this._query( type, item );
  };
});

function Query ( options ) {
  this.blog = options.blog;
  this.url = options.url;
  this.userpass = options.userpass;
  this.type = options.type;
  this.item = options.item || null;
  this.method = 'GET';
  this.body = null;
  this.filters = null;
  this.sorts = null;
  this.starts = null;
  this.limits = null;
}

Query.prototype.clone = function () {
  return new Query( this );
};

Query.prototype.filter = function () {
  this.filters = this.filters || [];
  this.filters.push( arguments );
  return this;
};

Query.prototype.sort = function () {
  this.sorts = arguments;
  return this;
};

Query.prototype.start = function ( starts ) {
  this.starts = starts;
  return this;
};

Query.prototype.limit = function ( limits ) {
  this.limits = limits;
  return this;
};

Query.prototype.update = function ( body ) {
  this.method = 'PUT';
  this.body = body;
  return this;
};

Query.prototype.create = function ( body ) {
  this.method = 'POST';
  this.body = body;
  return this;
};

Query.prototype.delete = function () {
  this.method = 'DELETE';
  return this;
};

Query.prototype.run = function ( cb ) {
  var options = { uri: '/' + this.type };

  if ( this.item instanceof Array ) {
    multi( this, cb );
    return;
  } else if ( this.item ) {
    options.uri = options.uri + '/' + this.item;
  }

  options.method = this.method;
  options.body = this.body;

  if (
    this.method === 'POST' &&
    ( this.type === 'posts' ||
      this.type === 'tags' )
     )
  {
    options.body.blog = this.blog;
  }

  options.body = JSON.stringify( options.body );

  var query = null;

  if ( this.sorts ) {
    query = {};
    query.sort = this.sorts[ 0 ];
    query.dir = this.sorts[ 1 ] || 'asc';
  }

  if ( this.filters ) {
    query = query || {};
    query.filter = JSON.stringify( this.filters );
  }

  if ( this.limits ) {
    query = query || {};
    query.limit = this.limits;
  }

  if ( this.starts ) {
    query = query || {};
    query.start = this.starts;
  }

  if ( query ) {
    query = '?' + querystring.stringify( query );
  } else {
    query = '';
  }

  options.uri = options.uri + query;

  api_request( this.url, this.userpass, options, function ( err, res, body ) {
    if ( err ) {
      cb( { code: 500, type: 'server', description: 'Service unavailable' } );
    } else {
      body = JSON.parse( body );
      if ( body.error ) {
        cb( body.error );
      } else {
        cb( null, body.data );
      }
    }
  });
};

function multi ( query, cb ) {
  var asyncs = {},
      items = query.item;

  items.forEach( function ( item ) {
    var newQuery = query.clone();
    newQuery.item = item;
    asyncs[ item ] = function ( cb ) { newQuery.run( cb ); };
  });

  async.parallel( asyncs, cb );
}

